#include <string>
#include <list>

#include <arc/registrar.h>


class RegistrarList;
class Registrar;

RegistrarList& GetRegistrarList(void);

class Registrar {
	friend class RegistrarList;
	public:
		RegistrarItem& GetItem(const std::string& id);
		void AddItem(RegistrarItem::GetItem_t r);
	private:
		std::string name;
		std::list<RegistrarItem::GetItem_t> items;
		Registrar(const char* name);
};

class RegistrarList {
	friend RegistrarList& GetRegistrarList(void);
	public:
		Registrar& GetRegistrar(const char* name);
	private:
		std::list<Registrar> regs;
		RegistrarList(void);
};

static char RegistrarMark[13];
#define RegistrarFootprint "ARCReGiStRaR"
static RegistrarList* registrar_list;

RegistrarList& GetRegistrarList(void) {
	if(memcmp(RegistrarMark,RegistrarFootprint,sizeof(RegistrarMark)) == 0) {
		return *registrar_list;
	};
	registrar_list=new RegistrarList;
	memcpy(RegistrarMark,RegistrarFootprint,sizeof(RegistrarMark));
	return *registrar_list;
}

RegistrarList::RegistrarList(void) {
}

Registrar& RegistrarList::GetRegistrar(const char* name) {
	for(std::list<Registrar>::iterator i = regs.begin();i!=regs.end();++i) {
		if(i->name == name) return *i;
	};
	std::list<Registrar>::iterator r = regs.insert(regs.end(),Registrar(name));
	return *r;
}

Registrar::Registrar(const char* name_):name(name_) {
}

RegistrarItem& Registrar::GetItem(const std::string& id) {
	for(std::list<RegistrarItem::GetItem_t>::iterator i = items.begin();i!=items.end();++i) {
		try {
			RegistrarItem& r = (*(*i))(id);
			return r;
		} catch(std::exception e) { };
	};
	throw RegistrarError("No item implements requested functionality");
}

void Registrar::AddItem(RegistrarItem::GetItem_t r) {
	items.insert(items.begin(),r);
}

RegistrarItem& RegistrarItem::GetItem(const std::string& name,const std::string& id) {
	return GetRegistrarList().GetRegistrar(name.c_str()).GetItem(id);
}

RegistrarItem::RegistrarItem(const std::string& name,RegistrarItem::GetItem_t get_item) {
	GetRegistrarList().GetRegistrar(name.c_str()).AddItem(get_item);
}

