/*
 * Created on Feb 9, 2006
 *
 */
package se.umu.jobsubmission.util;

import javax.xml.rpc.Stub;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.wsrf.WSRFConstants;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.ietf.jgss.GSSCredential;
import org.oasis.wsrf.properties.QueryExpressionType;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourceProperties_PortType;
import org.oasis.wsrf.properties.WSResourcePropertiesServiceAddressingLocator;

import se.umu.jobsubmission.util.QueryUtils;

/**
 * @author tordsson
 *
 * Utility class to simplify creation of QueryResourceProperties stubs
 * and request messages. 
 * 
 */
public class QueryUtils {

	/**
	 * Resolves stub for query-resource properties and sets security descriptor.
	 * @param serviceURL
	 * @param cred
	 * @return
	 * @throws Exception
	 */
	public static QueryResourceProperties_PortType getQueryPort(String serviceURL, 
			GlobusCredential cred) throws Exception {
		
		// Resolve service stub
		EndpointReferenceType epr = new EndpointReferenceType();
		epr.setAddress(new Address(serviceURL));
        WSResourcePropertiesServiceAddressingLocator locator =
            new WSResourcePropertiesServiceAddressingLocator();
        QueryResourceProperties_PortType queryPort =
        	locator.getQueryResourcePropertiesPort(epr);
        

        // Create and configure client security descriptor
		// This corresponds to -a -z none 
		ClientSecurityDescriptor secDesc = new ClientSecurityDescriptor();
		secDesc.setAnonymous(); 
		secDesc.setAuthz(NoAuthorization.getInstance()); 
		
		/*
		 Run from container
		GlobusCredential globusCredential = 
			JobSubmissionUtils.getDelegatedCredential(); 
		*/
		// Run as stand-alone client. 
		// GlobusCredential globusCredential = GlobusCredential.getDefaultCredential(); 
		
		GSSCredential credential = new GlobusGSSCredentialImpl(cred, 
				GSSCredential.INITIATE_AND_ACCEPT); 
		if(credential != null) {
			secDesc.setGSSCredential(credential); 
		}
		
		((Stub) queryPort)._setProperty(Constants.CLIENT_DESCRIPTOR, secDesc);
		
		return queryPort; 
	}
	
	public static QueryResourceProperties_Element getQuery(String expression) 
		throws Exception {
		
	    QueryResourceProperties_Element request = 
        	new QueryResourceProperties_Element();
        QueryExpressionType query = new QueryExpressionType();
        query.setDialect(WSRFConstants.XPATH_1_DIALECT);
        query.setValue(expression); 
        request.setQueryExpression(query);
        
        return request; 
	}
	
}
