package se.umu.jobsubmission.impl.jobsubmissionservice; 

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringWriter; 

import org.xml.sax.InputSource;

import org.apache.log4j.Logger;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.impl.PersistentReflectionResource;
import org.globus.wsrf.utils.FilePersistenceHelper;


import se.umu.jobsubmission.JobSubmissionPropertiesType;

public class JobSubmissionResource extends PersistentReflectionResource {

    private static Logger logger = Logger.getLogger(JobSubmissionResource.class); 
    
    public Class getResourceBeanClass() {
    	return JobSubmissionPropertiesType.class; 
    }
    
    // Main idea: As job id can contain characters illegal in filenames, 
    // store persitent resource using another value. 
    
    public synchronized void load(ResourceKey key) throws ResourceException {
    	
      	logger.debug("Loading resource"); 
    	try {
    		
    		// Get file storing persistent resource (actually, persistant RPs): 
    		// Ensure file name is positive... 
    		int keyValue = key.getValue().toString().hashCode(); 
    		if(keyValue < 0) 
    			keyValue *= -1; 
    		String fileKey = new Integer(keyValue).toString(); 
    		String configFilePath = getFileName(fileKey); 
    		logger.debug("Config file is: " + configFilePath); 
    		
    		// Deserialize resource properties from file. 
    		File f = new File(configFilePath); 
    		FileInputStream input = new FileInputStream(configFilePath); 
    		InputSource inputSource = new InputSource(input); 
    		JobSubmissionPropertiesType properties = 
    			(JobSubmissionPropertiesType) ObjectDeserializer.deserialize(
    					inputSource, 
						JobSubmissionPropertiesType.class); 
    		
    		// Initialize this resource from the deserialized properties. 
    		this.initialize(properties, 
    				JobSubmissionPropertiesType.getTypeDesc().getXmlType(),
					"" + properties.getSubmittedJob().hashCode()); 
    	} catch (Exception e) {
    		logger.error("Error loading resource: " + e.getMessage()); 
    	}
    }
    
    // Write to RPs temporary file, delete old serialized file 
    // and rename temporary file. 
    public synchronized void store() throws ResourceException {    	 
    	
    	try {
    		// Create tmp file. 
    		FilePersistenceHelper helper = new FilePersistenceHelper(
    				JobSubmissionPropertiesType.class, ".ser"); 
    		File tmpFile = File.createTempFile("jobsubmission", ".tmp", 
    				helper.getStorageDirectory());     		
    		
    		// Serialize properties and write them to tmp file
    		StringWriter sw = new StringWriter(); 
    		ObjectSerializer.serialize(sw, this.getResourceBean(), 
    				JobSubmissionPropertiesType.getTypeDesc().getXmlType()); 
    		String s = sw.toString();     		    		
    		FileOutputStream fos = new FileOutputStream(tmpFile); 
    		fos.write(s.getBytes()); 
    		fos.close();     		    		
    		
    		// Delete old file and move it to new, 
    		// Ensure that filename (integer) is not negative. 
    		int keyValue = this.getID().hashCode(); 
    		if(keyValue < 0)
    			keyValue *= -1;     		
    		//String key = new Integer (this.getID().hashCode()).toString(); 
    		String key = new Integer(keyValue).toString(); 
    		String fileName = getFileName(key); 
    		logger.debug("Storing resource in " + fileName); 
    		
    		File rpFile = new File(fileName); 
    		if(rpFile.exists()) {
    			rpFile.delete(); 
    		}
    		tmpFile.renameTo(rpFile); 
    		
    	} catch (Exception e) {
    		logger.error("Can not store resource properties: " 
    				+ e.getMessage()); 
    	}
    }
  
    /**
     * Remove file (jobid.hashCode().xml) storing resource properties. 
     */
    public void remove() {
    	
    	logger.debug("Removing resource"); 
    	File f = null; 
    	try {
    		String key = new Integer(this.getID().hashCode()).toString(); 
    		String fileName = getFileName(key);
    		f = new File(fileName); 
    		if(f.exists()) {
    			f.delete(); 
    		}
    	} catch (Exception e) {
    		logger.warn("Could not remove file " + ((f != null) ? 
    				f.getAbsolutePath() : "")); 
    	}
    	
    }
    
    /**
     * Gets path to file where RPs is/will be persisted. 
     * @param fileNameKey local file == resourceKey.hashCode
     * @return
     * @throws IOException if
     */
    public String getFileName(String fileNameKey) throws IOException {
    	
      	// Get config file ((resourcekey.hasCode()).xml)
		FilePersistenceHelper helper = 
			new FilePersistenceHelper(JobSubmissionPropertiesType.class, ".xml"); 
		String configFilePath = helper.getStorageDirectory().getAbsolutePath() 
				+ "/"
				+ fileNameKey 
				+ helper.getFileSuffix(); 
		
		return configFilePath;     	    	
    }            
}
